@@ -127,6 +127,9 @@ int fossil_islower(char c){ return c>='a' && c<='z'; }
 int fossil_isupper(char c){ return c>='A' && c<='Z'; }
 int fossil_isdigit(char c){ return c>='0' && c<='9'; }
 int fossil_isxdigit(char c){ return (c>='0' && c<='9') || (c>='a' && c<='f'); }
+int fossil_isXdigit(char c){
+   return (c>='0' && c<='9') || (c>='A' && c<='F') || (c>='a' && c<='f');
+}
 int fossil_tolower(char c){
   return fossil_isupper(c) ? c - 'A' + 'a' : c;
 }
@@ -664,7 +667,8 @@ void blob_dehttpize(Blob *pBlob){
 
 /*
 ** Extract N bytes from blob pFrom and use it to initialize blob pTo.
-** Return the actual number of bytes extracted.
+** Return the actual number of bytes extracted.  The cursor position
+** is advanced by the number of bytes extracted.
 **
 ** After this call completes, pTo will be an ephemeral blob.
 */
@@ -687,6 +691,53 @@ int blob_extract(Blob *pFrom, int N, Blob *pTo){
   return N;
 }
 
+/*
+** Extract N **lines** of text from blob pFrom beginning at the current
+** cursor position and use that text to initialize blob pTo.  Unlike the
+** blob_extract() routine,  the cursor position is unchanged.
+**
+** pTo is assumed to be uninitialized.
+**
+** After this call completes, pTo will be an ephemeral blob.
+*/
+int blob_extract_lines(Blob *pFrom, int N, Blob *pTo){
+  int i;
+  int mx;
+  int iStart;
+  int n;
+  const char *z;
+
+  blob_zero(pTo);
+  z = pFrom->aData;
+  i = pFrom->iCursor;
+  mx = pFrom->nUsed;
+  while( N>0 ){
+    while( i<mx && z[i]!='\n' ){ i++; }
+    if( i>=mx ) break;
+    i++;
+    N--;
+  }
+  iStart = pFrom->iCursor;
+  n = blob_extract(pFrom, i-pFrom->iCursor, pTo);
+  pFrom->iCursor = iStart;
+  return n;
+}
+
+/*
+** Return the number of lines of text in the blob.  If the last
+** line is incomplete (if it does not have a \n at the end) then
+** it still counts.
+*/
+int blob_linecount(Blob *p){
+  int n = 0;
+  int i;
+  for(i=0; i<p->nUsed; i++){
+    if( p->aData[i]=='\n' ) n++;
+  }
+  if( p->nUsed>0 && p->aData[p->nUsed-1]!='\n' ) n++;
+  return n;
+}
+
 /*
 ** Rewind the cursor on a blob back to the beginning.
 */
@@ -1945,7 +1996,7 @@ void blob_to_utf8_no_bom(Blob *pBlob, int useMbcs){
     blob_reset(pBlob);
     blob_set_dynamic(pBlob, zUtf8);
   }else if( useMbcs && invalid_utf8(pBlob) ){
-#if defined(_WIN32) || defined(__CYGWIN__)
+#if defined(_WIN32)
     zUtf8 = fossil_mbcs_to_utf8(blob_str(pBlob));
     blob_reset(pBlob);
     blob_append(pBlob, zUtf8, -1);
