@@ -83,14 +83,10 @@ void branch_new(void){
   const char *zDateOvrd; /* Override date string */
   const char *zUserOvrd; /* Override user name */
   int isPrivate = 0;     /* True if the branch should be private */
-  int bAutoColor = 0;    /* Value of "--bgcolor" is "auto" */
 
   noSign = find_option("nosign","",0)!=0;
+  if( find_option("nosync",0,0) ) g.fNoSync = 1;
   zColor = find_option("bgcolor","c",1);
-  if( fossil_strncmp(zColor, "auto", 4)==0 ) {
-    bAutoColor = 1;
-    zColor = 0;
-  }
   isPrivate = find_option("private",0,0)!=0;
   zDateOvrd = find_option("date-override",0,1);
   zUserOvrd = find_option("user-override",0,1);
@@ -154,7 +150,6 @@ void branch_new(void){
   /* Add the symbolic branch name and the "branch" tag to identify
   ** this as a new branch */
   if( content_is_private(rootid) ) isPrivate = 1;
-  if( isPrivate && zColor==0 && !bAutoColor) zColor = "#fec084";
   if( zColor!=0 ){
     blob_appendf(&branch, "T *bgcolor * %F\n", zColor);
   }
@@ -605,7 +600,7 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 ** Run various subcommands to manage branches of the open repository or
 ** of the repository identified by the -R or --repository option.
 **
-** >  fossil branch close|reopen ?OPTIONS? BRANCH-NAME ?...BRANCH-NAMES?
+** > fossil branch close|reopen ?OPTIONS? BRANCH-NAME ?...BRANCH-NAMES?
 **
 **       Adds or cancels the "closed" tag to one or more branches.
 **       It accepts arbitrary unambiguous symbolic names but
@@ -619,22 +614,22 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 **         --date-override DATE  DATE to use instead of 'now'
 **         --user-override USER  USER to use instead of the current default
 **
-** >  fossil branch current
+** > fossil branch current
 **
 **        Print the name of the branch for the current check-out
 **
-** >  fossil branch hide|unhide ?OPTIONS? BRANCH-NAME ?...BRANCH-NAMES?
+** > fossil branch hide|unhide ?OPTIONS? BRANCH-NAME ?...BRANCH-NAMES?
 **
 **       Adds or cancels the "hidden" tag for the specified branches or
 **       or check-in IDs. Accepts the same options as the close
 **       subcommand.
 **
-** >  fossil branch info BRANCH-NAME
+** > fossil branch info BRANCH-NAME
 **
 **        Print information about a branch
 **
-** >  fossil branch list|ls ?OPTIONS? ?GLOB?
-** >  fossil branch lsh ?OPTIONS? ?LIMIT?
+** > fossil branch list|ls ?OPTIONS? ?GLOB?
+** > fossil branch lsh ?OPTIONS? ?LIMIT?
 **
 **        List all branches.
 **
@@ -647,8 +642,8 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 **          -r               Reverse the sort order
 **          -t               Show recently changed branches first
 **          --self           List only branches where you participate
-**          --username USER  List only branches where USER participate
-**          --users N        List up to N users partipiating
+**          --username USER  List only branches where USER participates
+**          --users N        List up to N users participating
 **
 **        The current branch is marked with an asterisk.  Private branches are
 **        marked with a hash sign.
@@ -660,25 +655,25 @@ static void branch_cmd_close(int nStartAtArg, int fClose){
 **        but no GLOB argument.  All other options are supported, with -t being
 **        an implied no-op.
 **
-** >  fossil branch new BRANCH-NAME BASIS ?OPTIONS?
+** > fossil branch new BRANCH-NAME BASIS ?OPTIONS?
 **
 **        Create a new branch BRANCH-NAME off of check-in BASIS.
 **
+**        This command is available for people who want to create a branch
+**        in advance.  But  the use of this command is discouraged.  The
+**        preferred idiom in Fossil is to create new branches at the point
+**        of need, using the "--branch NAME" option to the "fossil commit"
+**        command.
+**
 **        Options:
 **          --private             Branch is private (i.e., remains local)
 **          --bgcolor COLOR       Use COLOR instead of automatic background
-**                                ("auto" lets Fossil choose it automatically,
-**                                even for private branches)
-**          --nosign              Do not sign contents on this branch
+**          --nosign              Do not sign the manifest for the check-in
+**                                that creates this branch
+**          --nosync              Do not auto-sync prior to creating the branch
 **          --date-override DATE  DATE to use instead of 'now'
 **          --user-override USER  USER to use instead of the current default
 **
-**        DATE may be "now" or "YYYY-MM-DDTHH:MM:SS.SSS". If in
-**        year-month-day form, it may be truncated, the "T" may be
-**        replaced by a space, and it may also name a timezone offset
-**        from UTC as "-HH:MM" (westward) or "+HH:MM" (eastward).
-**        Either no timezone suffix or "Z" means UTC.
-**
 ** Options:
 **    -R|--repository REPO       Run commands on repository REPO
 */
@@ -877,7 +872,8 @@ static void new_brlist_page(void){
     char *zAge = human_readable_age(rNow - rMtime);
     sqlite3_int64 iMtime = (sqlite3_int64)(rMtime*86400.0);
     if( zMergeTo && zMergeTo[0]==0 ) zMergeTo = 0;
-    if( zBgClr == 0 ){
+    if( zBgClr ) zBgClr = reasonable_bg_color(zBgClr, 0);
+    if( zBgClr==0 ){
       if( zBranch==0 || strcmp(zBranch,"trunk")==0 ){
         zBgClr = 0;
       }else{
