@@ -207,7 +207,7 @@ void page_dir(void){
       isSymbolicCI = (sqlite3_strnicmp(zUuid, zCI, strlen(zCI))!=0);
       isBranchCI = branch_includes_uuid(zCI, zUuid);
       if( bDocDir ) zCI = mprintf("%S", zUuid);
-      Th_Store("current_checkin", zCI);
+      Th_StoreUnsafe("current_checkin", zCI);
     }else{
       zCI = 0;
     }
@@ -749,7 +749,7 @@ void page_tree(void){
   /* If a regular expression is specified, compile it */
   zRE = P("re");
   if( zRE ){
-    re_compile(&pRE, zRE, 0);
+    fossil_re_compile(&pRE, zRE, 0);
     zREx = mprintf("&re=%T", zRE);
   }
   cgi_check_for_malice();
@@ -773,7 +773,7 @@ void page_tree(void){
                          " FROM event WHERE objid=%d", rid);
       isSymbolicCI = (sqlite3_strnicmp(zUuid, zCI, strlen(zCI)) != 0);
       isBranchCI = branch_includes_uuid(zCI, zUuid);
-      Th_Store("current_checkin", zCI);
+      Th_StoreUnsafe("current_checkin", zCI);
     }else{
       zCI = 0;
     }
@@ -862,13 +862,20 @@ void page_tree(void){
   }else{
     Stmt q;
     db_prepare(&q,
+      "WITH mx(fnid,fid,mtime) AS (\n"
+      "  SELECT fnid, fid, max(event.mtime)\n"
+      "    FROM mlink, event\n"
+      "   WHERE event.objid=mlink.mid\n"
+      "   GROUP BY 1\n"
+      ")\n"
       "SELECT\n"
-      "    (SELECT name FROM filename WHERE filename.fnid=mlink.fnid),\n"
-      "    (SELECT uuid FROM blob WHERE blob.rid=mlink.fid),\n"
-      "    (SELECT size FROM blob WHERE blob.rid=mlink.fid),\n"
-      "    max(event.mtime)\n"
-      "  FROM mlink JOIN event ON event.objid=mlink.mid\n"
-      " GROUP BY mlink.fnid\n"
+      "  filename.name,\n"
+      "  blob.uuid,\n"
+      "  blob.size,\n"
+      "  mx.mtime\n"
+      "FROM mx\n"
+      " LEFT JOIN filename ON filename.fnid=mx.fnid\n"
+      " LEFT JOIN blob ON blob.rid=mx.fid\n"
       " ORDER BY 1 COLLATE uintnocase;");
     while( db_step(&q)==SQLITE_ROW ){
       const char *zName = db_column_text(&q, 0);
@@ -1033,7 +1040,8 @@ static const char zComputeFileAgeSetup[] =
 @   fid INTEGER,
 @   mid INTEGER,
 @   mtime DATETIME,
-@   pathname TEXT
+@   pathname TEXT,
+@   uuid TEXT
 @ );
 @ CREATE VIRTUAL TABLE IF NOT EXISTS temp.foci USING files_of_checkin;
 ;
@@ -1045,8 +1053,9 @@ static const char zComputeFileAgeRun[] =
 @              SELECT plink.pid
 @                FROM ckin, plink
 @               WHERE plink.cid=ckin.x)
-@ INSERT OR IGNORE INTO fileage(fnid, fid, mid, mtime, pathname)
-@   SELECT filename.fnid, mlink.fid, mlink.mid, event.mtime, filename.name
+@ INSERT OR IGNORE INTO fileage(fnid, fid, mid, mtime, pathname, uuid)
+@   SELECT filename.fnid, mlink.fid, mlink.mid, event.mtime, filename.name,
+@          foci.uuid
 @     FROM foci, filename, blob, mlink, event
 @    WHERE foci.checkinID=:ckin
 @      AND foci.filename GLOB :glob
@@ -1157,7 +1166,6 @@ void fileage_page(void){
   double baseTime;
   login_check_credentials();
   if( !g.perm.Read ){ login_needed(g.anon.Read); return; }
-  if( exclude_spiders() ) return;
   zName = P("name");
   if( zName==0 ) zName = "tip";
   rid = symbolic_name_to_rid(zName, "ci");
@@ -1253,20 +1261,3 @@ void fileage_page(void){
   db_finalize(&q2);
   style_finish_page();
 }
-
-/*
-** WEBPAGE: files
-**
-** Show files as a flat table.  If the ci=LABEL query parameter is provided,
-** then show all the files in the specified check-in.  Without the ci= query
-** parameter show all files across all check-ins.
-**
-** Query parameters:
-**
-**    name=PATH        Directory to display.  Optional
-**    ci=LABEL         Show only files in this check-in.  Optional.
-**    re=REGEXP        Show only files matching REGEXP.  Optional.
-*/
-void files_page(void){
-  return;
-}
