@@ -827,9 +827,9 @@ void test_ci_mini_cmd(void){
   zFilename = g.argv[2];
   cimi.zFilename = mprintf("%/", zAsFilename ? zAsFilename : zFilename);
   cimi.filePerm = file_perm(zFilename, ExtFILE);
-  cimi.zUser = mprintf("%s", zUser ? zUser : login_name());
+  cimi.zUser = fossil_strdup(zUser ? zUser : login_name());
   if(zDate){
-    cimi.zDate = mprintf("%s", zDate);
+    cimi.zDate = fossil_strdup(zDate);
   }
   if(zRevision==0 || zRevision[0]==0){
     if(g.localOpen/*check-out*/){
@@ -930,7 +930,7 @@ static char *fileedit_file_uuid(char const *zFilename,
              "WHERE filename=%Q %s AND checkinID=%d",
              zFilename, filename_collation(), vid);
   if(SQLITE_ROW==db_step(&stmt)){
-    zFileUuid = mprintf("%s",db_column_text(&stmt, 0));
+    zFileUuid = fossil_strdup(db_column_text(&stmt, 0));
     if(pFilePerm){
       *pFilePerm = mfile_permstr_int(db_column_text(&stmt, 1));
     }
@@ -1189,7 +1189,7 @@ static int fileedit_setup_cimi_from_p(CheckinMiniInfo * p, Blob * pErr,
     }
     fail((pErr,"Missing required 'filename' parameter."));
   }
-  p->zFilename = mprintf("%s",zFlag);
+  p->zFilename = fossil_strdup(zFlag);
 
   if(0==fileedit_is_editable(p->zFilename)){
     rc = 403;
@@ -1250,7 +1250,7 @@ static int fileedit_setup_cimi_from_p(CheckinMiniInfo * p, Blob * pErr,
   }
   zFlag = P("comment_mimetype");
   if(zFlag){
-    p->zCommentMimetype = mprintf("%s",zFlag);
+    p->zCommentMimetype = fossil_strdup(zFlag);
     zFlag = 0;
   }
 #define p_int(K) atoi(PD(K,"0"))
@@ -1286,7 +1286,7 @@ static int fileedit_setup_cimi_from_p(CheckinMiniInfo * p, Blob * pErr,
   ** TODO?: date-override date selection field. Maybe use
   ** an input[type=datetime-local].
   */
-  p->zUser = mprintf("%s",g.zLogin);
+  p->zUser = fossil_strdup(g.zLogin);
   return 0;
 end_fail:
 #undef fail
