@@ -20,6 +20,7 @@
 #include "config.h"
 #include "path.h"
 #include <assert.h>
+#include <math.h>
 
 #if INTERFACE
 /* Nodes for the paths through the DAG.
@@ -29,12 +30,14 @@ struct PathNode {
   u8 fromIsParent;         /* True if pFrom is the parent of rid */
   u8 isPrim;               /* True if primary side of common ancestor */
   u8 isHidden;             /* Abbreviate output in "fossil bisect ls" */
+  char *zBranch;           /* Branch name for this node.  Might be NULL */
+  double mtime;            /* Date/time of this check-in */
   PathNode *pFrom;         /* Node we came from */
   union {
-    PathNode *pPeer;       /* List of nodes of the same generation */
+    double rCost;          /* Cost of getting to this node from pStart */
     PathNode *pTo;         /* Next on path from beginning to end */
   } u;
-  PathNode *pAll;        /* List of all nodes */
+  PathNode *pAll;          /* List of all nodes */
 };
 #endif
 
@@ -42,14 +45,16 @@ struct PathNode {
 ** Local variables for this module
 */
 static struct {
-  PathNode *pCurrent;   /* Current generation of nodes */
+  PQueue pending;       /* Nodes pending review for inclusion in the graph */
   PathNode *pAll;       /* All nodes */
-  Bag seen;             /* Nodes seen before */
   int nStep;            /* Number of steps from first to last */
   int nNotHidden;       /* Number of steps not counting hidden nodes */
+  int brCost;           /* Extra cost for moving to a different branch */
+  int revCost;          /* Extra cost for changing directions */
   PathNode *pStart;     /* Earliest node */
   PathNode *pEnd;       /* Most recent */
 } path;
+static int path_debug = 0;  /* Flag to enable debugging */
 
 /*
 ** Return the first (last) element of the computed path.
@@ -68,21 +73,67 @@ int path_length(void){ return path.nStep; }
 int path_length_not_hidden(void){ return path.nNotHidden; }
 
 /*
-** Create a new node
+** Used for debugging only.
+**
+** Given a RID, return the ISO date/time string and branch for the
+** corresponding check-in.  Memory is held locally and is overwritten
+** with each call.
+*/
+char *path_rid_desc(int rid){
+  static Stmt q;
+  static char *zDesc = 0;
+  db_static_prepare(&q, 
+    "SELECT concat(strftime('%%Y%%m%%d%%H%%M',event.mtime),'/',value)"
+    "  FROM event, tagxref"
+    " WHERE event.objid=:rid"
+    "   AND tagxref.rid=:rid"
+    "   AND tagxref.tagid=%d"
+    "   AND tagxref.tagtype>0",
+    TAG_BRANCH
+  );
+  fossil_free(zDesc);
+  db_bind_int(&q, ":rid", rid);
+  if( db_step(&q)==SQLITE_ROW ){
+    zDesc = fossil_strdup(db_column_text(&q,0));
+  }
+  db_reset(&q);
+  return zDesc ? zDesc : "???";
+}
+
+/*
+** Create a new node and insert it into the path.pending queue.
 */
 static PathNode *path_new_node(int rid, PathNode *pFrom, int isParent){
   PathNode *p;
 
   p = fossil_malloc( sizeof(*p) );
   memset(p, 0, sizeof(*p));
+  p->pAll = path.pAll;
+  path.pAll = p;
   p->rid = rid;
   p->fromIsParent = isParent;
   p->pFrom = pFrom;
-  p->u.pPeer = path.pCurrent;
-  path.pCurrent = p;
-  p->pAll = path.pAll;
-  path.pAll = p;
-  bag_insert(&path.seen, rid);
+  p->u.rCost = pFrom ? pFrom->u.rCost : 0.0;
+  if( path.brCost ){
+    p->zBranch = branch_of_rid(rid);
+    p->mtime = mtime_of_rid(rid, 0.0);
+    if( pFrom ){
+      p->u.rCost +=  fabs(pFrom->mtime - p->mtime);
+      if( fossil_strcmp(p->zBranch, pFrom->zBranch)!=0 ){
+        p->u.rCost += path.brCost;
+      }
+    }
+  }else{
+    /* When brCost==0, we try to minimize the number of nodes
+    ** along the path.  The cost is just the number of nodes back
+    ** to the start.  We do not need to know the branch name nor
+    ** the mtime */
+    p->u.rCost += 1.0;
+  }
+  if( path_debug ){
+    fossil_print("PUSH %-50s cost = %g\n", path_rid_desc(p->rid), p->u.rCost);
+  }
+  pqueuex_insert_ptr(&path.pending, (void*)p, p->u.rCost);
   return p;
 }
 
@@ -94,9 +145,10 @@ void path_reset(void){
   while( path.pAll ){
     p = path.pAll;
     path.pAll = p->pAll;
+    fossil_free(p->zBranch);
     fossil_free(p);
   }
-  bag_clear(&path.seen);
+  pqueuex_clear(&path.pending);
   memset(&path, 0, sizeof(path));
 }
 
@@ -130,13 +182,15 @@ PathNode *path_shortest(
   int iTo,            /* Path ends here */
   int directOnly,     /* No merge links if true */
   int oneWayOnly,     /* Parent->child only if true */
-  Bag *pHidden        /* Hidden nodes */
+  Bag *pHidden,       /* Hidden nodes */
+  int branchCost      /* Add extra cost to changing branches */
 ){
   Stmt s;
-  PathNode *pPrev;
+  Bag seen;
   PathNode *p;
 
   path_reset();
+  path.brCost = branchCost;
   path.pStart = path_new_node(iFrom, 0, 0);
   if( iTo==iFrom ){
     path.pEnd = path.pStart;
@@ -154,40 +208,42 @@ PathNode *path_shortest(
     db_prepare(&s,
         "SELECT cid, 1 FROM plink WHERE pid=:pid AND isprim "
         "UNION ALL "
-        "SELECT pid, 0 FROM plink WHERE cid=:pid AND isprim"
+        "SELECT pid, 0 FROM plink WHERE :back AND cid=:pid AND isprim"
     );
   }else{
     db_prepare(&s,
         "SELECT cid, 1 FROM plink WHERE pid=:pid "
         "UNION ALL "
-        "SELECT pid, 0 FROM plink WHERE cid=:pid"
+        "SELECT pid, 0 FROM plink WHERE :back AND cid=:pid"
     );
   }
-  while( path.pCurrent ){
-    path.nStep++;
-    pPrev = path.pCurrent;
-    path.pCurrent = 0;
-    while( pPrev ){
-      db_bind_int(&s, ":pid", pPrev->rid);
-      while( db_step(&s)==SQLITE_ROW ){
-        int cid = db_column_int(&s, 0);
-        int isParent = db_column_int(&s, 1);
-        if( bag_find(&path.seen, cid) ) continue;
-        p = path_new_node(cid, pPrev, isParent);
-        if( pHidden && bag_find(pHidden,cid) ) p->isHidden = 1;
-        if( cid==iTo ){
-          db_finalize(&s);
-          path.pEnd = p;
-          path_reverse_path();
-          for(p=path.pStart->u.pTo; p; p=p->u.pTo ){
-            if( !p->isHidden ) path.nNotHidden++;
-          }
-          return path.pStart;
-        }
-      }
-      db_reset(&s);
-      pPrev = pPrev->u.pPeer;
+  bag_init(&seen);
+  while( (p = pqueuex_extract_ptr(&path.pending))!=0 ){
+    if( path_debug ){
+      printf("PULL %s %g\n", path_rid_desc(p->rid), p->u.rCost);
     }
+    if( p->rid==iTo ){
+      db_finalize(&s);
+      path.pEnd = p;
+      path_reverse_path();
+      for(p=path.pStart->u.pTo; p; p=p->u.pTo ){
+        if( !p->isHidden ) path.nNotHidden++;
+      }
+      return path.pStart;
+    }
+    if( bag_find(&seen, p->rid) ) continue;
+    bag_insert(&seen, p->rid);
+    db_bind_int(&s, ":pid", p->rid);
+    if( !oneWayOnly ) db_bind_int(&s, ":back", !p->fromIsParent);
+    while( db_step(&s)==SQLITE_ROW ){
+      int cid = db_column_int(&s, 0);
+      int isParent = db_column_int(&s, 1);
+      PathNode *pNew;
+      if( bag_find(&seen, cid) ) continue;
+      pNew = path_new_node(cid, p, isParent);
+      if( pHidden && bag_find(pHidden,cid) ) pNew->isHidden = 1;
+    }
+    db_reset(&s);
   }
   db_finalize(&s);
   path_reset();
@@ -218,6 +274,18 @@ PathNode *path_next(void){
   return p;
 }
 
+/*
+** Return the branch for a path node.
+**
+** Storage space is managed by the path subsystem.  The returned value
+** is valid until the path is reset.
+*/
+const char *path_branch(PathNode *p){
+  if( p==0 ) return 0;
+  if( p->zBranch==0 ) p->zBranch = branch_of_rid(p->rid);
+  return p->zBranch;
+}
+
 /*
 ** Return an estimate of the number of comparisons remaining in order
 ** to bisect path.  This is based on the log2() of path.nStep.
@@ -240,7 +308,7 @@ void path_shortest_stored_in_ancestor_table(
   PathNode *pPath;
   int gen = 0;
   Stmt ins;
-  pPath = path_shortest(cid, origid, 1, 0, 0);
+  pPath = path_shortest(cid, origid, 1, 0, 0, 0);
   db_multi_exec(
     "CREATE TEMP TABLE IF NOT EXISTS ancestor("
     "  rid INT UNIQUE,"
@@ -263,10 +331,15 @@ void path_shortest_stored_in_ancestor_table(
 /*
 ** COMMAND: test-shortest-path
 **
-** Usage: %fossil test-shortest-path ?--no-merge? VERSION1 VERSION2
+** Usage: %fossil test-shortest-path [OPTIONS] VERSION1 VERSION2
 **
-** Report the shortest path between two check-ins.  If the --no-merge flag
-** is used, follow only direct parent-child paths and omit merge links.
+** Report the shortest path between two check-ins.  Options:
+**
+**    --branch-cost N    Additional cost N for changing branches
+**    --debug            Show debugging output
+**    --one-way          One-way forwards in time, parent->child only
+**    --no-merge         Follow only direct parent-child paths and omit
+**                       merge links.
 */
 void shortest_path_test_cmd(void){
   int iFrom;
@@ -275,33 +348,25 @@ void shortest_path_test_cmd(void){
   int n;
   int directOnly;
   int oneWay;
+  const char *zBrCost;
 
   db_find_and_open_repository(0,0);
   directOnly = find_option("no-merge",0,0)!=0;
   oneWay = find_option("one-way",0,0)!=0;
+  zBrCost = find_option("branch-cost",0,1);
+  if( find_option("debug",0,0)!=0 ) path_debug = 1;
   if( g.argc!=4 ) usage("VERSION1 VERSION2");
   iFrom = name_to_rid(g.argv[2]);
   iTo = name_to_rid(g.argv[3]);
-  p = path_shortest(iFrom, iTo, directOnly, oneWay, 0);
+  p = path_shortest(iFrom, iTo, directOnly, oneWay, 0,
+                    zBrCost ? atoi(zBrCost) : 0);
   if( p==0 ){
     fossil_fatal("no path from %s to %s", g.argv[1], g.argv[2]);
   }
   for(n=1, p=path.pStart; p; p=p->u.pTo, n++){
-    char *z;
-    z = db_text(0,
-      "SELECT substr(uuid,1,12) || ' ' || datetime(mtime)"
-      "  FROM blob, event"
-      " WHERE blob.rid=%d AND event.objid=%d AND event.type='ci'",
-      p->rid, p->rid);
-    fossil_print("%4d: %5d %s", n, p->rid, z);
-    fossil_free(z);
-    if( p->u.pTo ){
-      fossil_print(" is a %s of\n",
-                   p->u.pTo->fromIsParent ? "parent" : "child");
-    }else{
-      fossil_print("\n");
-    }
+    fossil_print("%4d: %s\n", n, path_rid_desc(p->rid));
   }
+  path_debug = 0;
 }
 
 /*
@@ -310,7 +375,7 @@ void shortest_path_test_cmd(void){
 */
 int path_common_ancestor(int iMe, int iYou){
   Stmt s;
-  PathNode *pPrev;
+  PathNode *pThis;
   PathNode *p;
   Bag me, you;
 
@@ -325,41 +390,36 @@ int path_common_ancestor(int iMe, int iYou){
   bag_insert(&me, iMe);
   bag_init(&you);
   bag_insert(&you, iYou);
-  while( path.pCurrent ){
-    pPrev = path.pCurrent;
-    path.pCurrent = 0;
-    while( pPrev ){
-      db_bind_int(&s, ":cid", pPrev->rid);
-      while( db_step(&s)==SQLITE_ROW ){
-        int pid = db_column_int(&s, 0);
-        if( bag_find(pPrev->isPrim ? &you : &me, pid) ){
-          /* pid is the common ancestor */
-          PathNode *pNext;
-          for(p=path.pAll; p && p->rid!=pid; p=p->pAll){}
-          assert( p!=0 );
-          pNext = p;
-          while( pNext ){
-            pNext = p->pFrom;
-            p->pFrom = pPrev;
-            pPrev = p;
-            p = pNext;
-          }
-          if( pPrev==path.pStart ) path.pStart = path.pEnd;
-          path.pEnd = pPrev;
-          path_reverse_path();
-          db_finalize(&s);
-          return pid;
-        }else if( bag_find(&path.seen, pid) ){
-          /* pid is just an alternative path on one of the legs */
-          continue;
+  while( (pThis = pqueuex_extract_ptr(&path.pending))!=0 ){
+    db_bind_int(&s, ":cid", pThis->rid);
+    while( db_step(&s)==SQLITE_ROW ){
+      int pid = db_column_int(&s, 0);
+      if( bag_find(pThis->isPrim ? &you : &me, pid) ){
+        /* pid is the common ancestor */
+        PathNode *pNext;
+        for(p=path.pAll; p && p->rid!=pid; p=p->pAll){}
+        assert( p!=0 );
+        pNext = p;
+        while( pNext ){
+          pNext = p->pFrom;
+          p->pFrom = pThis;
+          pThis = p;
+          p = pNext;
         }
-        p = path_new_node(pid, pPrev, 0);
-        p->isPrim = pPrev->isPrim;
-        bag_insert(pPrev->isPrim ? &me : &you, pid);
+        if( pThis==path.pStart ) path.pStart = path.pEnd;
+        path.pEnd = pThis;
+        path_reverse_path();
+        db_finalize(&s);
+        return pid;
+      }else if( bag_find(pThis->isPrim ? &me : &you, pid) ){
+        /* pid is just an alternative path to a node we've already visited */
+        continue;
       }
-      db_reset(&s);
-      pPrev = pPrev->u.pPeer;
+      p = path_new_node(pid, pThis, 0);
+      p->isPrim = pThis->isPrim;
+      bag_insert(pThis->isPrim ? &me : &you, pid);
     }
+    db_reset(&s);
   }
   db_finalize(&s);
   path_reset();
@@ -455,7 +515,7 @@ void find_filename_changes(
   }
   if( iFrom==iTo ) return;
   path_reset();
-  p = path_shortest(iFrom, iTo, 1, revOK==0, 0);
+  p = path_shortest(iFrom, iTo, 1, revOK==0, 0, 0);
   if( p==0 ) return;
   path_reverse_path();
   db_prepare(&q1,
@@ -470,6 +530,14 @@ void find_filename_changes(
       continue;
     }
     db_bind_int(&q1, ":mid", p->rid);
+    if( zDebug ){
+      fossil_print("%s check-in %.16z %z rid %d\n",
+         zDebug,
+         db_text(0, "SELECT uuid FROM blob WHERE rid=%d", p->rid),
+         db_text(0, "SELECT date(mtime) FROM event WHERE objid=%d", p->rid),
+         p->rid
+      );
+    }
     while( db_step(&q1)==SQLITE_ROW ){
       fnid = db_column_int(&q1, 1);
       pfnid = db_column_int(&q1, 0);
@@ -483,9 +551,8 @@ void find_filename_changes(
         pfnid = t;
       }
       if( zDebug ){
-        fossil_print("%s at %d%s %.10z: %d[%z] -> %d[%z]\n",
-           zDebug, p->rid, p->fromIsParent ? ">" : "<",
-           db_text(0, "SELECT uuid FROM blob WHERE rid=%d", p->rid),
+        fossil_print("%s %d[%z] -> %d[%z]\n",
+           zDebug,
            pfnid,
            db_text(0, "SELECT name FROM filename WHERE fnid=%d", pfnid),
            fnid,
@@ -537,6 +604,7 @@ void find_filename_changes(
       fossil_free(pChng);
     }
   }
+  path_reset();
 }
 
 /*
