@@ -1434,7 +1434,7 @@ static int setParameterCmd(
   if( argc!=3 ){
     return Th_WrongNumArgs(interp, "setParameter NAME VALUE");
   }
-  cgi_replace_parameter(mprintf("%s", argv[1]), mprintf("%s", argv[2]));
+  cgi_replace_parameter(fossil_strdup(argv[1]), fossil_strdup(argv[2]));
   return TH_OK;
 }
 
@@ -2147,7 +2147,7 @@ static int regexpCmd(
   if( nArg+2!=argc ){
     return Th_WrongNumArgs(interp, REGEXP_WRONGNUMARGS);
   }
-  zErr = re_compile(&pRe, argv[nArg], noCase);
+  zErr = fossil_re_compile(&pRe, argv[nArg], noCase);
   if( !zErr ){
     Th_SetResultInt(interp, re_match(pRe,
         (const unsigned char *)argv[nArg+1], TH1_LEN(argl[nArg+1])));
@@ -2204,7 +2204,7 @@ static int httpCmd(
   }
   zRegexp = db_get("th1-uri-regexp", 0);
   if( zRegexp && zRegexp[0] ){
-    const char *zErr = re_compile(&pRe, zRegexp, 0);
+    const char *zErr = fossil_re_compile(&pRe, zRegexp, 0);
     if( zErr ){
       Th_SetResult(interp, zErr, -1);
       return TH_ERROR;
@@ -2967,7 +2967,7 @@ int Th_RenderToBlob(const char *z, Blob * pOut, u32 mFlags){
       if( z[0] ){ z += 6; }
       i = 0;
     }else{
-      i++;
+      i += strcspn(&z[i+1], "<$") + 1;
     }
   }
   if( rc==TH_ERROR ){
