@@ -310,6 +310,7 @@ proc get_all_settings {} {
       crnl-glob \
       default-csp \
       default-perms \
+      default-skin \
       diff-binary \
       diff-command \
       dont-commit \
@@ -333,6 +334,7 @@ proc get_all_settings {} {
       fileedit-glob \
       forbid-delta-manifests \
       forum-close-policy \
+      forum-title \
       gdiff-command \
       gmerge-command \
       hash-digits \
@@ -356,10 +358,13 @@ proc get_all_settings {} {
       pgp-command \
       preferred-diff-type \
       proxy \
+      raw-bgcolor \
       redirect-to-https \
       relative-paths \
       repo-cksum \
       repolist-skin \
+      robot-restrict \
+      robots-txt \
       safe-html \
       self-pw-reset \
       self-register \
@@ -371,8 +376,15 @@ proc get_all_settings {} {
       th1-setup \
       th1-uri-regexp \
       ticket-default-report \
+      timeline-hard-newlines \
+      timeline-plaintext \
+      timeline-truncate-at-blank \
+      timeline-tslink-info \
+      timeline-utc \
       user-color-map \
+      verify-comments \
       uv-sync \
+      vuln-report \
       web-browser]
 
   fossil test-th-eval "hasfeature legacyMvRm"
